#include "bits/stdc++.h"
#include "testlib.h"

using namespace std;

using ll = long long;

const int MAX_C = 1'000'000'000;

int sign(ll x) {
    if (x == 0) return 0;
    return (x < 0 ? -1 : 1);
}

ll sq(ll a) {
    return a * a;
}

#define vec point

struct point {
    int x = 0, y = 0;

    point operator+(const point &other) const {
        return point({x + other.x, y + other.y});
    }

    point operator-(const point &other) const {
        return point({x - other.x, y - other.y});
    }

    ll operator*(const point &other) const {
        return 1ll * x * other.x + 1ll * y * other.y;
    }

    ll operator^(const point &other) const {
        return 1ll * x * other.y - 1ll * y * other.x;
    }

    bool operator<(const point &other) const {
        if (sign(x - other.x) == 0) {
            return sign(y - other.y) < 0;
        } else {
            return sign(x - other.x) < 0;
        }
    }

    bool operator==(const point &other) const {
        return sign(x - other.x) == 0 && sign(y - other.y) == 0;
    }

    point perp() const {
        return point({-y, x});
    }

    long double len() const {
        return sqrtl(sq(x) + sq(y));
    }

    long long len2() const {
        return sq(x) + sq(y);
    }

    friend std::ostream &operator<<(std::ostream &os, const point &p) {
        os << "[x, y]: " << fixed << p.x << ' ' << p.y;
        return os;
    }
};

bool cmp(const pair<vec, int> &a, const pair<vec, int> &b) {
    int s = sign(a.first ^ b.first);
    if (s > 0) return true;
    else if (s == 0) return a.first.len2() < b.first.len2();
    else return false;
}

void sorting(vector<pair<vec, int>> &vectors) {
    vector<pair<vec, int>> up, dw;
    for (auto &v: vectors) {
        if (v.first.y > 0 || (v.first.y >= 0 && v.first.x >= 0)) {
            up.push_back(v);
        } else {
            dw.push_back(v);
        }
    }

    sort(up.begin(), up.end(), cmp);
    sort(dw.begin(), dw.end(), cmp);
    for (auto &i: dw) {
        up.push_back(i);
    }
    vectors = up;
}

bool in_hull(vector<point> &hull, point pt) {
    int ok = 0;
    for (int i = 0; i < hull.size(); i++) {
        vec a = hull[(i + 1) % hull.size()] - hull[i];
        if (sign(a ^ (pt - hull[i])) == 1) {
            ok |= 1;
        } else if (sign(a ^ (pt - hull[i])) == -1) {
            ok |= 2;
        } else {
            ok |= 3;
        }
    }
    return ok != 3;
}


bool in_triangle(point a, point b, point c, point fc) {
    vector<point> pt = {a, b, c};
    return in_hull(pt, fc);
}

bool check_points(vector<point> &hull, point c, vector<point> &in) {
    vector<pair<vec, int>> pts;
    pts.reserve(hull.size() + in.size());
    for (auto i: hull) {
        pts.push_back({i - c, 1});
    }
    for (auto i: in) {
        if (i == c) return false;
        pts.push_back({i - c, 2});
    }
    sorting(pts);
    int n = pts.size();
    for (int i = 0; i < n;) {
        int j = i;
        int ok = 0;
        while (j < n + i && sign(pts[j % n].first ^ pts[i].first) == 0) {
            ok |= pts[j % n].second;
            j++;
        }
        if (ok == 3) return false;
        i = j;
    }
    for (int i = 0; i < n;) {
        if (pts[i].second != 1) {
            i++;
            continue;
        }
        int j = i + 1;
        while (j < n + i && pts[j % n].second != 1) {
            j++;
        }
        for (int k = i + 1; k < j; k++) {
            if (!in_triangle(c, c + pts[i].first, c + pts[j % n].first, c + pts[k % n].first)) {
                return false;
            }
        }
        i = j;
    }
    return true;
}

bool check_rotation(vector<point> &hull) {
    for (int i = 0; i < hull.size(); i++) {
        point a = hull[i];
        point b = hull[(i + 1) % hull.size()];
        point c = hull[(i + 2) % hull.size()];
        if (sign((b - a) ^ (c - a)) < 0) return false;
        else if (sign((b - a) ^ (c - a)) == 0 && sign((b - a) * (c - a)) != 1) {
            return false;
        }
    }
    return true;
}

const int groups = 7;
const int ns[groups] = {100, 3, 18, 18, 5000, 5000, 10000};
const int qs[groups] = {100, 0, 1, 1'000'000, 1, 5000, 1'000'000};
const int ms[groups] = {100, 3, 18, 18, 5000, 100'000, 100'000};

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);
    int group = 1;
    if (!validator.group().empty()) {
        group = stoi(validator.group());
    } else {
        throw std::runtime_error("empty group");
    }
    int n = inf.readInt(3, ns[group], "n");
    inf.readSpace();
    int m = inf.readInt(0, ms[group], "m");
    inf.readSpace();
    int q = inf.readInt(0, qs[group], "q");
    inf.readEoln();
    point c;
    c.x = inf.readInt(-MAX_C + 1, MAX_C - 1, "c.x");
    inf.readSpace();
    c.y = inf.readInt(-MAX_C + 1, MAX_C - 1, "c.y");
    inf.readEoln();
    vector<point> hull(n), pts(m);
    for (auto &[x, y]: hull) {
        x = inf.readInt(-MAX_C, MAX_C, "x_hull");
        inf.readSpace();
        y = inf.readInt(-MAX_C, MAX_C, "y_hull");
        inf.readEoln();
    }
    ensuref(check_rotation(hull), "wrong convex hull rotation");
    ensuref(in_hull(hull, c), "wrong central point: central point does not lie inside the convex hull");
    set<point> use;
    for (int i = 0; i < m; i++) {
        int x, y;
        x = inf.readInt(-MAX_C + 1, MAX_C - 1, "x_pts");
        inf.readSpace();
        y = inf.readInt(-MAX_C + 1, MAX_C - 1, "y_pts");
        inf.readEoln();
        pts[i] = {x, y};
        ensuref(use.find(pts[i]) == use.end(), "internal points are not unique");
        use.insert(pts[i]);
    }
    pts.reserve(m + q);
    for (int i = 0; i < q; i++) {
        char t = inf.readChar();
        ensuref(t == '+' || t == '-', "wrong query format on query %d", i + 1);
        inf.readSpace();
        point pt;
        pt.x = inf.readInt(-MAX_C, MAX_C, "x_pts");
        inf.readSpace();
        pt.y = inf.readInt(-MAX_C, MAX_C, "y_pts");
        inf.readEoln();
        if (t == '+') {
            pts.push_back(pt);
            ensuref(use.find(pt) == use.end(), "internal points after query %d are not unique", i + 1);
            use.insert(pt);
        } else {
            ensuref(use.find(pt) != use.end(), "wrong query format on query %d: this point does not exist", i + 1);
            use.erase(use.find(pt));
        }
    }
    ensuref(check_points(hull, c, pts), "wrong internal points");
    inf.readEof();
}
