#include <bits/stdc++.h>
using namespace std;
 
using ll = long long;
using ld = long double;
 
#define all(a) begin(a), end(a)
#define len(a) int((a).size())
 
struct point {
    ll x;
    ll y;
 
    ll operator%(const point& other) const {
        return x * other.y - y * other.x;
    }
 
    point operator-(const point& other) const {
        return {x - other.x, y - other.y};
    }
 
    bool above() const {
        if (y == 0) {
            return x > 0;
        }
        return y > 0;
    }
};
 
struct segtree {
    struct node {
        int cnt{0};
        int pref{0};
        int suf{0};
        int len{1};
        int ans{0};
 
        static node merge(const node& left, const node& right, const vector<int>& nim) {
            node res;
            res.pref = left.pref;
            if (left.pref == left.len) {
                res.pref += right.pref;
            }
            res.suf = right.suf;
            if (right.suf == right.len) {
                res.suf += left.suf;
            }
            res.len = left.len + right.len;
            res.ans = left.ans ^ right.ans ^ nim[left.suf] ^ nim[right.pref] ^ nim[left.suf + right.pref];
            return res;
        }
    };
 
    int n;
    vector<node> tree;
    vector<int> nim;
 
    segtree(int n) : n(n), tree(4 * n), nim(n + 1) {
        build(1, 0, n);
 
        vector<int> used(n + 2, 0);
        for (int k = 1; k <= n; k++) {
            used[nim[k - 1]] = k;
            for (int left = 0; left + 2 <= k; left++) {
                used[nim[left] ^ nim[k - left - 2]] = k;
            }
            while (used[nim[k]] == k) {
                nim[k]++;
            }
        }
    }
 
    void build(int v, int vl, int vr) {
        if (vr - vl == 1) {
            return;
        }
        int vm = (vl + vr) >> 1;
        build(v << 1, vl, vm);
        build(v << 1 | 1, vm, vr);
        tree[v] = node::merge(tree[v << 1], tree[v << 1 | 1], nim);
    }
 
    void update(int pos, int d) {
        update(1, 0, n, pos, d);
    }
 
    void update(int v, int vl, int vr, int pos, int d) {
        if (vr - vl == 1) {
            assert(vl == pos);
            tree[v].cnt += d;
            assert(tree[v].cnt >= 0);
            if (tree[v].cnt == 0) {
                tree[v].pref = tree[v].suf = 0;
            } else {
                tree[v].pref = tree[v].suf = 1;
            }
            tree[v].ans = nim[tree[v].cnt > 0];
            return;
        }
 
        int vm = (vl + vr) >> 1;
        if (pos < vm) {
            update(v << 1, vl, vm, pos, d);
        } else {
            update(v << 1 | 1, vm, vr, pos, d);
        }
        tree[v] = node::merge(tree[v << 1], tree[v << 1 | 1], nim);
    }
 
    bool alice_wins() const {
        if (tree[1].pref == n) {
            return true;
        }
        return (tree[1].ans ^ nim[tree[1].pref] ^ nim[tree[1].suf] ^ nim[tree[1].pref + tree[1].suf]) != 0;
    }
};
 
int main() {
    cin.tie(nullptr)->sync_with_stdio(false);
 
    int n, m, q;
    cin >> n >> m >> q;
    point c;
    cin >> c.x >> c.y;
 
    vector<point> poly(n);
    for (auto& [x, y] : poly) {
        cin >> x >> y;
    }
 
    auto compare = [&](const point& a, const point& b) {
        if (a.above() != b.above()) {
            return a.above();
        }
        return a % b > 0;
    };
 
    int smallest = 0;
    for (int i = 1; i < n; i++) {
        if (compare(poly[i] - c, poly[smallest] - c)) {
            smallest = i;
        }
    }
    rotate(poly.begin(), poly.begin() + smallest, poly.end());
    
    segtree tree(n);
 
    auto update = [&](const point& p, int d) {
        int l = 0;
        int r = n;
        if (compare(p - c, poly[0] - c)) {
            l = n - 1;
        }
        while (r - l > 1) {
            int mid = (l + r) >> 1;
            (compare(poly[mid] - c, p - c) ? l : r) = mid;
        }
        tree.update(l, d);
    };
 
    while (m--) {
        point p;
        cin >> p.x >> p.y;
        update(p, 1);
    }
 
    auto print_ans = [&]() {
        cout << (tree.alice_wins() ? "Alice" : "Bob") << '\n';
    };
 
    print_ans();
    while (q--) {
        char type;
        point p;
        cin >> type >> p.x >> p.y;
        update(p, type == '+' ? 1 : -1);
        print_ans();
    }
}
